#[macro_use]
extern crate structopt;

#[macro_use]
extern crate derive_builder;

mod controller;
mod gen;
mod listener;
mod model;
mod opts;

use crate::gen::gen_events;
use crate::listener::*;
use crate::model::{ECCKey, Peer};
use controller::Userspace;
use model::WireguardController;
use opts::Opts;
use std::collections::HashMap;
use std::io;
use std::thread::sleep;
use std::time::{Duration, Instant};
use structopt::StructOpt;

fn listeners(opts: &Opts) -> Vec<Box<EventListener>> {
    let mut listeners: Vec<Box<EventListener>> = Vec::with_capacity(2);
    if let Some(events) = opts.events.clone() {
        listeners.push(Box::new(ScriptListener::new(events)))
    }
    if opts.log {
        listeners.push(Box::new(LogListener));
    }
    listeners
}

fn main() -> io::Result<()> {
    let opts = Opts::from_args();
    let mut controller: Box<WireguardController> = Box::new(Userspace::new(opts.socket.clone()));
    let interval = Duration::from_millis(opts.poll);
    let timeout = Duration::from_secs(opts.timeout);
    let listeners = listeners(&opts);

    println!(
        "Polling {} every {:?}",
        opts.socket.to_str().unwrap(),
        interval
    );
    let mut peers_last: Option<HashMap<ECCKey, Peer>> = None;
    loop {
        let now = Instant::now();
        let peers = controller.peers()?;
        /*println!("Connected peers:");
        for peer in peers {
            let peer = peer?;
            if peer
                .last_handshake
                .map(|h| h.elapsed().unwrap() < timeout)
                .unwrap_or(false)
            {
                println!("/\\{:?}  {}",(timeout - peer.last_handshake.unwrap().elapsed().unwrap()), peer);
            }
        }*/
        let peers = peers
            .map(|peer| peer.map(|peer_ok| (peer_ok.key.clone(), peer_ok)))
            .collect::<io::Result<HashMap<_, _>>>()?;
        if let Some(ref mut peers_last) = peers_last {
      
                gen_events(&peers, &peers_last, &listeners, timeout, interval);
            
            *peers_last = peers;
        } else {
            peers_last = Some(peers);
        }
        let pause = interval - now.elapsed();
        dbg!(interval - pause);
        sleep(if pause > interval / 2 {
            pause
        } else {
            interval
        });
    }
    Ok(())
}
